@@ -1,6 +1,6 @@
 /* BLINKENLIGHTS PROXY
- * version 0.97 date 2005-12-29
- * Copyright (C) 2003-2005 Stefan Schuermans <1stein@schuermans.info>
+ * version 0.98 date 2006-07-27
+ * Copyright (C) 2003-2006 Stefan Schuermans <1stein@schuermans.info>
  * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
  */
 
@@ -11,10 +11,12 @@
 #include <fcntl.h>
 #include <memory.h>
 #include <signal.h>
+#include <unistd.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/select.h>
 #include <sys/time.h>
+#include <sys/un.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <errno.h>
@@ -26,7 +28,7 @@
 #include "output.h"
 #include "tools.h"
 
-#define BL_PROXY_VERSION_STR "version 0.97 date 2005-12-29"
+#define BL_PROXY_VERSION_STR "version 0.98 date 2006-07-27"
 
 //default listen address for control connections
 #define listen_af "in"
@@ -125,8 +127,10 @@ int main( int arg_cnt, char * * args )
   unsigned int stdin_pos, input_file_pos, ctrl_conn_pos[ctrl_conn_cnt_max];
   struct sockaddr_in ctrl_sock_addr_in, ctrl_conn_sock_addr_in;
   struct sockaddr_in6 ctrl_sock_addr_in6, ctrl_conn_sock_addr_in6;
+  struct sockaddr_un ctrl_sock_addr_un, ctrl_conn_sock_addr_un;
   struct sockaddr * p_ctrl_sock_addr, * p_ctrl_conn_sock_addr;
   int ctrl_sock_addr_len, ctrl_conn_sock_addr_len;
+  char * unlink_ctrl_listen_sock;
   st_select_fd_sets fd_sets;
   struct timeval timeout;
   st_out_info output_file_out_info, out_info;
@@ -190,11 +194,10 @@ int main( int arg_cnt, char * * args )
     printf( "\n"
             "BLINKENLIGHTS PROXY\n"
             BL_PROXY_VERSION_STR"\n"
-            "Copyright (C) 2003-2005 Stefan Schuermans <1stein@schuermans.info>\n"
+            "Copyright (C) 2003-2006 Stefan Schuermans <1stein@schuermans.info>\n"
             "Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html\n"
 	    "\n" );
 
-
   //parameter error
   if( param_err )
   {
@@ -202,7 +205,7 @@ int main( int arg_cnt, char * * args )
             "\n"
             "options: \"-i <input-file>\" (defaults to \"%s\")\n"
             "         \"-o <output-file>\" (defaults to \"%s\")\n"
-            "         \"-f <listen-address-family>\" (\"in\" or \"in6\", defaults to \""listen_af"\")\n"
+            "         \"-f <listen-address-family>\" (\"none\", \"in\", \"in6\" or \"unix\", defaults to \""listen_af"\")\n"
             "         \"-a <listen-address>\" (defaults to \""listen_addr"\")\n"
             "         \"-p <listen-port>\" (defaults to \""listen_port"\")\n"
             "         \"-l\" enable local command input\n"
@@ -236,26 +239,21 @@ int main( int arg_cnt, char * * args )
     output_file_out_info = *p_out_none;
   }
 
-  //create socket for control connections
-  ctrl_listen_sock = socket( AF_INET, SOCK_STREAM, 0 );
-  if( ctrl_listen_sock == -1 )
-  {
-    if( input_file != -1 )
-      close( input_file );
-    if( output_file != -1 )
-      close( output_file );
-    printf( "could not create socket for control connections: %s\n\n", strerror( errno ) );
-    return -1;
-  }
-
   //generate address to listen on for control connections
-  if( strcasecmp( p_af, "in" ) == 0 )
+  if( strcasecmp( p_af, "none" ) == 0 )
+  {
+    p_ctrl_sock_addr = NULL;
+	ctrl_sock_addr_len = 0;
+    p_ctrl_conn_sock_addr = NULL;
+    ctrl_conn_sock_addr_len = 0;
+	unlink_ctrl_listen_sock = NULL;
+  }
+  else if( strcasecmp( p_af, "in" ) == 0 )
   {
     ctrl_sock_addr_in.sin_family = AF_INET;
     ctrl_sock_addr_in.sin_port = htons( (uint16_t)atoi( p_port ) );
     if( ! inet_pton( AF_INET, p_addr, &ctrl_sock_addr_in.sin_addr ) )
     {
-      close( ctrl_listen_sock );
       if( input_file != -1 )
         close( input_file );
       if( output_file != -1 )
@@ -267,6 +265,7 @@ int main( int arg_cnt, char * * args )
     ctrl_sock_addr_len = sizeof( ctrl_sock_addr_in );
     p_ctrl_conn_sock_addr = (struct sockaddr *)&ctrl_conn_sock_addr_in;
     ctrl_conn_sock_addr_len = sizeof( ctrl_conn_sock_addr_in );
+	unlink_ctrl_listen_sock = NULL;
   }
   else if( strcasecmp( p_af, "in6" ) == 0 )
   {
@@ -274,7 +273,6 @@ int main( int arg_cnt, char * * args )
     ctrl_sock_addr_in6.sin6_port = htons( atoi( p_port ) );
     if( ! inet_pton( AF_INET6, p_addr, &ctrl_sock_addr_in6.sin6_addr ) )
     {
-      close( ctrl_listen_sock );
       if( input_file != -1 )
         close( input_file );
       if( output_file != -1 )
@@ -286,48 +284,96 @@ int main( int arg_cnt, char * * args )
     ctrl_sock_addr_len = sizeof( ctrl_sock_addr_in6 );
     p_ctrl_conn_sock_addr = (struct sockaddr *)&ctrl_conn_sock_addr_in6;
     ctrl_conn_sock_addr_len = sizeof( ctrl_conn_sock_addr_in6 );
+	unlink_ctrl_listen_sock = NULL;
+  }
+  else if( strcasecmp( p_af, "unix" ) == 0 )
+  {
+    ctrl_sock_addr_un.sun_family = AF_UNIX;
+	if( strlen( p_addr ) > sizeof( ctrl_sock_addr_un.sun_path ) - 1 )
+	{
+      if( input_file != -1 )
+        close( input_file );
+      if( output_file != -1 )
+        close( output_file );
+      printf( "invalid UNIX-address (too long): %s\n\n", p_addr );
+      return -1;
+	}
+    strcpy( ctrl_sock_addr_un.sun_path, p_addr );
+    p_ctrl_sock_addr = (struct sockaddr *)&ctrl_sock_addr_un;
+    ctrl_sock_addr_len = sizeof( ctrl_sock_addr_un );
+    p_ctrl_conn_sock_addr = (struct sockaddr *)&ctrl_conn_sock_addr_un;
+    ctrl_conn_sock_addr_len = sizeof( ctrl_conn_sock_addr_un );
+	unlink_ctrl_listen_sock = p_addr;
   }
   else
   {
-    close( ctrl_listen_sock );
     if( input_file != -1 )
       close( input_file );
     if( output_file != -1 )
       close( output_file );
-    printf( "invalid address family \"%s\" for control connections (try \"in\" or \"in6\")\n\n", p_af );
+    printf( "invalid address family \"%s\" for control connections (try \"none\", \"in\", \"in6\" or \"unix\")\n\n", p_af );
     return -1;
   }
 
-  //bind socket for control connections
-  if( bind( ctrl_listen_sock, p_ctrl_sock_addr, ctrl_sock_addr_len ) == -1 )
+  //create socket for control connections
+  if( p_ctrl_sock_addr == NULL || p_ctrl_conn_sock_addr == NULL )
   {
-    close( ctrl_listen_sock );
-    if( input_file != -1 )
-      close( input_file );
-    if( output_file != -1 )
-      close( output_file );
-    printf( "could not bind socket to \"%s %s %s\" for control connections: %s\n\n", p_af, p_addr, p_port, strerror( errno ) );
-    return -1;
+    ctrl_listen_sock = -1; //control connections disabled
+  }
+  else
+  {
+    ctrl_listen_sock = socket( p_ctrl_sock_addr->sa_family, SOCK_STREAM, 0 );
+    if( ctrl_listen_sock == -1 )
+    {
+      if( input_file != -1 )
+        close( input_file );
+      if( output_file != -1 )
+        close( output_file );
+      printf( "could not create socket for control connections: %s\n\n", strerror( errno ) );
+      return -1;
+    }
   }
 
-  //listen on socket for control connections
-  if( listen( ctrl_listen_sock, ctrl_conn_cnt_max ) == -1 )
+  if( ctrl_listen_sock != -1 )
   {
-    close( ctrl_listen_sock );
-    if( input_file != -1 )
-      close( input_file );
-    if( output_file != -1 )
-      close( output_file );
-    printf( "could not listen on socket for control connections: %s\n\n", strerror( errno ) );
-    return -1;
+    //bind socket for control connections
+    if( bind( ctrl_listen_sock, p_ctrl_sock_addr, ctrl_sock_addr_len ) == -1 )
+    {
+      close( ctrl_listen_sock );
+      if( input_file != -1 )
+        close( input_file );
+      if( output_file != -1 )
+        close( output_file );
+      printf( "could not bind socket to \"%s %s %s\" for control connections: %s\n\n", p_af, p_addr, p_port, strerror( errno ) );
+      return -1;
+    }
+
+    //listen on socket for control connections
+    if( listen( ctrl_listen_sock, ctrl_conn_cnt_max ) == -1 )
+    {
+      close( ctrl_listen_sock );
+      if( unlink_ctrl_listen_sock != NULL )
+        unlink( unlink_ctrl_listen_sock );
+      if( input_file != -1 )
+        close( input_file );
+      if( output_file != -1 )
+        close( output_file );
+      printf( "could not listen on socket for control connections: %s\n\n", strerror( errno ) );
+      return -1;
+    }
+    for( i = 0; i < ctrl_conn_cnt_max; i++ )
+      ctrl_conn_socks[i] = -1;
   }
-  for( i = 0; i < ctrl_conn_cnt_max; i++ )
-    ctrl_conn_socks[i] = -1;
 
   //initialize global stuff
   if( global_init( p_out_stdout ) != 0 )
   {
-    close( ctrl_listen_sock );
+    if( ctrl_listen_sock != -1 )
+    {
+      close( ctrl_listen_sock );
+      if( unlink_ctrl_listen_sock != NULL )
+        unlink( unlink_ctrl_listen_sock );
+    }
     if( input_file != -1 )
       close( input_file );
     if( output_file != -1 )
@@ -377,25 +423,29 @@ int main( int arg_cnt, char * * args )
       if( input_file + 1 > fd_sets.n )
         fd_sets.n = input_file + 1;
     }
-    //add control connection sockets
-    for( i = 0; i < ctrl_conn_cnt_max; i++ ) //check if place for new connection
-      if( ctrl_conn_socks[i] == -1 )
-        break;
-    if( i < ctrl_conn_cnt_max ) //place for new connection is available
+    //control connections enabled
+    if( ctrl_listen_sock != -1 )
     {
-      FD_SET( ctrl_listen_sock, &fd_sets.read_fd_set );
-      if( ctrl_listen_sock + 1 > fd_sets.n )
-        fd_sets.n = ctrl_listen_sock + 1;
-    }
-    for( i = 0; i < ctrl_conn_cnt_max; i++ ) //established connections
-    {
-      if( ctrl_conn_socks[i] != -1 )
+      //add control connection sockets
+      for( i = 0; i < ctrl_conn_cnt_max; i++ ) //check if place for new connection
+        if( ctrl_conn_socks[i] == -1 )
+          break;
+      if( i < ctrl_conn_cnt_max ) //place for new connection is available
       {
-        FD_SET( ctrl_conn_socks[i], &fd_sets.read_fd_set );
-        if( ctrl_conn_socks[i] + 1 > fd_sets.n )
-          fd_sets.n = ctrl_conn_socks[i] + 1;
+        FD_SET( ctrl_listen_sock, &fd_sets.read_fd_set );
+        if( ctrl_listen_sock + 1 > fd_sets.n )
+          fd_sets.n = ctrl_listen_sock + 1;
       }
-    }
+      for( i = 0; i < ctrl_conn_cnt_max; i++ ) //established connections
+      {
+        if( ctrl_conn_socks[i] != -1 )
+        {
+          FD_SET( ctrl_conn_socks[i], &fd_sets.read_fd_set );
+          if( ctrl_conn_socks[i] + 1 > fd_sets.n )
+            fd_sets.n = ctrl_conn_socks[i] + 1;
+        }
+      }
+    } //control connections enabled
 
     //initialize timeout
     timeout.tv_sec = 0;
@@ -433,52 +483,56 @@ int main( int arg_cnt, char * * args )
           output_file_out_info = *p_out_none;
         }
       }
-      //pending connection on control socket
-      if( FD_ISSET( ctrl_listen_sock, &fd_sets.read_fd_set ) )
+      //control connections enabled
+      if( ctrl_listen_sock != -1 )
       {
-        //search free place
-        for( i = 0; i < ctrl_conn_cnt_max; i++ )
-          if( ctrl_conn_socks[i] == -1 )
-            break;
-        if( i < ctrl_conn_cnt_max )
+        //pending connection on control socket
+        if( FD_ISSET( ctrl_listen_sock, &fd_sets.read_fd_set ) )
         {
-          //accept connection
-          len = ctrl_conn_sock_addr_len;
-          ctrl_conn_socks[i] = accept( ctrl_listen_sock, p_ctrl_conn_sock_addr, &len );
-          if( ctrl_conn_socks[i] != -1 )
+          //search free place
+          for( i = 0; i < ctrl_conn_cnt_max; i++ )
+            if( ctrl_conn_socks[i] == -1 )
+              break;
+          if( i < ctrl_conn_cnt_max )
           {
-            //clear buffer
-            ctrl_conn_pos[i] = 0;
-	    //output message and first prompt
-            out_info = out_info_fd( ctrl_conn_socks[i] );
-            out_str( &out_info, "BLINKENLIGHTS PROXY\n"
-                                BL_PROXY_VERSION_STR"\n"
-                                "\n"
-                                "bl_proxy> " );
-            out_flush( &out_info );
-          }
-        }
-      }
-      //input on control connection
-      for( i = 0; i < ctrl_conn_cnt_max; i++ )
-      {
-        if( ctrl_conn_socks[i] != -1 )
-        {
-          if( FD_ISSET( ctrl_conn_socks[i], &fd_sets.read_fd_set ) )
-          {
-            //process input
-            out_info = out_info_fd( ctrl_conn_socks[i] );
-            conn_end = process_input( ctrl_conn_socks[i], ctrl_conn_buffer[i], sizeof( ctrl_conn_buffer[i] ), &ctrl_conn_pos[i], &out_info, 1 ); //returns !=0 if connection should end
-            //close connection
-            if( conn_end )
+            //accept connection
+            len = ctrl_conn_sock_addr_len;
+            ctrl_conn_socks[i] = accept( ctrl_listen_sock, p_ctrl_conn_sock_addr, &len );
+            if( ctrl_conn_socks[i] != -1 )
             {
-              shutdown( ctrl_conn_socks[i], SHUT_RDWR );
-              close( ctrl_conn_socks[i] );
-              ctrl_conn_socks[i] = -1;
+              //clear buffer
+              ctrl_conn_pos[i] = 0;
+              //output message and first prompt
+              out_info = out_info_fd( ctrl_conn_socks[i] );
+              out_str( &out_info, "BLINKENLIGHTS PROXY\n"
+                                  BL_PROXY_VERSION_STR"\n"
+                                  "\n"
+                                  "bl_proxy> " );
+              out_flush( &out_info );
             }
           }
         }
-      }
+        //input on control connection
+        for( i = 0; i < ctrl_conn_cnt_max; i++ )
+        {
+          if( ctrl_conn_socks[i] != -1 )
+          {
+            if( FD_ISSET( ctrl_conn_socks[i], &fd_sets.read_fd_set ) )
+            {
+              //process input
+              out_info = out_info_fd( ctrl_conn_socks[i] );
+              conn_end = process_input( ctrl_conn_socks[i], ctrl_conn_buffer[i], sizeof( ctrl_conn_buffer[i] ), &ctrl_conn_pos[i], &out_info, 1 ); //returns !=0 if connection should end
+              //close connection
+              if( conn_end )
+              {
+                shutdown( ctrl_conn_socks[i], SHUT_RDWR );
+                close( ctrl_conn_socks[i] );
+                ctrl_conn_socks[i] = -1;
+              }
+            }
+          }
+        }
+      } //control connections enabled
     } //at least one event
   } //for( end = 0; ! end; )
   if( ! quiet )
@@ -497,7 +551,12 @@ int main( int arg_cnt, char * * args )
     }
   }
   //close listening socket for control connections
-  close( ctrl_listen_sock );
+  if( ctrl_listen_sock != -1 )
+  {
+    close( ctrl_listen_sock );
+    if( unlink_ctrl_listen_sock != NULL )
+      unlink( unlink_ctrl_listen_sock );
+  }
   //close input and output file
   if( input_file != -1 )
     close( input_file );
